#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2009,2010,2011  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
from .utils import get_paths, require_path

################################################################################
# Exceptions
################################################################################

class XDGError(Exception):
  """Base class for exceptions in this module."""
  pass



class UnsetEnvVarError(XDGError):
  """Exception raised when expected environment variable is unset."""

  def __init__(self, var):
    self.var = var

  def __str__(self):
    return self.__class__.__name__ + ': "' + self.var + '" is not set'



class InvalidRuntimeDir(XDGError):
  """Exception raised when the XDG_RUNTIME_DIR does not adhere to the specification."""

  def __init__(self, var):
    self.var = var

  def __str__(self):
    return self.__class__.__name__ + ': "' + self.var + '" does not adhere to the specification.'



################################################################################
# XDG Base Directory Specification
# Implemented version: 0.7
# http://standards.freedesktop.org/basedir-spec/basedir-spec-0.7.html
#
# Compare to the latest version and update if necessary:
# http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html
################################################################################

def get_data_home(subpath=None):
  """Get the base directory for user-specific data files.

  If a subpath is given, it will be appended to the base directory.
  """
  path = os.getenv('XDG_DATA_HOME')
  if not path:
    path = os.path.join(require_path('HOME'), '.local/share')
  if subpath:
    path = os.path.join(path, subpath)
  return path



def get_config_home(subpath=None):
  """Get the base directory for user-specific configuration files.

  If a subpath is given, it will be appended to the base directory.
  """
  path = os.getenv('XDG_CONFIG_HOME')
  if not path:
    path = os.path.join(require_path('HOME'), '.config')
  if subpath:
    path = os.path.join(path, subpath)
  return path



def get_data_dirs(subpath=None):
  """Get a list of base directories for data files.

  If a subpath is given, it will be appended to the base directories.
  """
  paths = get_paths('XDG_DATA_DIRS')
  if not paths:
    paths = ['/usr/local/share/', '/usr/share/']
  if subpath:
    paths = [os.path.join(path, subpath) for path in paths]
  return paths



def get_config_dirs(subpath=None):
  """Get a list of base directories for data files.

  If a subpath is given, it will be appended to the base directories.
  """
  paths = get_paths('XDG_CONFIG_DIRS')
  if not paths:
    paths = ['/etc/xdg']
  if subpath:
    paths = [os.path.join(path, subpath) for path in paths]
  return paths



def get_cache_home(subpath=None):
  """Get the base directory for user-specific cached data.

  If a subpath is given, it will be appended to the base directory.
  """
  path = os.getenv('XDG_CACHE_HOME')
  if not path:
    path = os.path.join(require_path('HOME'), '.cache')
  if subpath:
    path = os.path.join(path, subpath)
  return path



def get_runtime_dir(subpath=None):
  """Get the base directory for user-specific runtime files.

  If a subpath is given, it will be appended to the base directory.
  """
  # This checks some parts of the specification such as existence,
  # user-ownership and existence.
  # It does not check support for the following:
  #   * AF_UNIX socket
  #   * symbolic link
  #   * hard link
  #   * proper permission
  #   * file locking
  #   * sparse file
  #   * memory mapping
  #   * file change notifications
  #   * reliable hard link count
  #   * unrestricted file name character set
  path = os.getenv('XDG_RUNTIME_HOME')
  if not path or not os.path.isdir(path):
    raise UnsetEnvVarError('XDG_RUNTIME_HOME')
  else:
    stat = os.stat(path)
    if stat.st_uid != os.getuid() or (stat.st_mod & 0o7777) != 0o0600:
      raise UnsetEnvVarError('XDG_RUNTIME_HOME')
  if subpath:
    path = os.path.join(path, subpath)
  return path



def search_data_dirs(subpath=None, include_home=True):
  """Search for existing data directories, or an existing subpath thereof.

  Returns a generator.
  """
  if include_home:
    path = get_data_home(subpath)
    if path and os.path.exists(path):
      yield path
  for path in get_data_dirs(subpath):
    if path and os.path.exists(path):
      yield path



def search_config_dirs(subpath=None, include_home=True):
  """Search for existing configuration directories, or an existing subpath
  thereof.

  Returns a generator.
  """
  if include_home:
    path = get_config_home(subpath)
    if path and os.path.exists(path):
      yield path
  for path in get_config_dirs(subpath):
    if path and os.path.exists(path):
      yield path
